#!/bin/bash

set -e

basedir="/tmp/phg_obj"
arch=`uname -i`

if test "$arch" = "i386"; then
    long="long long"
else
    long="long"
fi
for m in mpich2 openmpi; do
    module purge
    if ! module load ${m}-${arch}; then
	echo 1>&2 "Failed to load module \"${m}-${arch}\". Abort."
	exit 1
    fi
    dir0="${basedir}/${m}-${arch}"
    if ! mkdir -p "$dir0"; then
	echo 1>&2 "Failed to create directory \"$dir0\". Abort"
	exit 1
    fi
    if ! env LANG=C mpicc -v >"${dir0}/MPI-version.txt" 2>&1; then
	echo 1>&2 "Failed to create file \"${dir0}/MPI-version.txt\". Abort"
	exit 1
    fi
    for f in "double" "long double" "__float128"; do
	for i in "int" "$long"; do
	    dir="`echo ${dir0}/${f}_${i} | sed -e 's/  */-/g'`"
	    if ! mkdir -p "$dir"; then
		echo 1>&2 "Failed to create directory \"$dir\". Abort"
		exit 1
	    fi
	    echo 1>&2 "Building in \"$dir\" ..."
	    echo 1>&2 "    configure --with-float=\"$f\" --with-int=\"$i\" ..."
	    if ! ./configure --with-float="$f" --with-int="$i" \
		--disable-solver >/dev/null 2>&1; then
		echo 1>&2 "Failed to configure phg. Abort."
		exit 1
	    fi
	    /bin/rm -f src/{refine.o,coarsen.o}
	    cd src
	    echo 1>&2 "    make refine.o coarsen.o ..."
	    if ! make -s refine.o coarsen.o; then
		echo 1>&2 "Failed to compile. Abort."
		exit 1
	    fi
	    if ! mv refine.o coarsen.o "${dir}/."; then
		echo 1>&2 "Failed to move files to \"${dir}\". Abort."
		exit 1
	    fi
	    cd ..
	done
    done
done

echo 1>&2 "obj files successfully built in \"$basedir\"."
